/*
  GRANITE DATA SERVICES
  Copyright (C) 2007-2008 ADEQUATE SYSTEMS SARL

  This file is part of Granite Data Services.

  Granite Data Services is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 3 of the License, or (at your
  option) any later version.

  Granite Data Services is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, see <http://www.gnu.org/licenses/>.
*/

package org.granite.tide.validation;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;

import org.granite.context.GraniteContext;
import org.granite.tide.validators.EntityValidator;
import org.granite.tide.validators.InvalidValue;


public class BeanValidation implements EntityValidator {
    
    private static final String KEY = BeanValidation.class.getName() + ".validators";
    
    private ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
    
    
    public BeanValidation() {
        GraniteContext gc = GraniteContext.getCurrentInstance();
        Map<Class<?>, Validator> validators = new ConcurrentHashMap<Class<?>, Validator>(20);
        gc.getApplicationMap().put(KEY, validators);
    }
    
    
    @SuppressWarnings("unchecked")
    public InvalidValue[] getPotentialInvalidValues(Class<?> entityClass, String propertyName, Object value) {
        GraniteContext gc = GraniteContext.getCurrentInstance();
        Map<Class<?>, Validator> validators = (Map<Class<?>, Validator>)gc.getApplicationMap().get(KEY);
        Validator validator = validators.get(entityClass);
        if (validator == null) {
            validator = validatorFactory.getValidator();
            validators.put(entityClass, validator);
        }
        
        Set<?> constraintViolations = validator.validateValue(entityClass, propertyName, value);
        return convertConstraintViolations((Set<ConstraintViolation<?>>)constraintViolations);
    }


    public static InvalidValue[] convertConstraintViolations(Set<ConstraintViolation<?>> constraintViolations) {
        InvalidValue[] converted = new org.granite.tide.validators.InvalidValue[constraintViolations.size()];
        int i = 0;
        for (ConstraintViolation<?> cv : constraintViolations) {
            if (cv.getRootBean() == null) {
                converted[i++] = new InvalidValue(
                    cv.getRootBeanClass(),
                    cv.getPropertyPath().toString(),
                    cv.getInvalidValue(),
                    cv.getMessage()
                );
            }
            else {
                converted[i++] = new InvalidValue(
                    cv.getRootBean(),
                    cv.getLeafBean() != null ? cv.getLeafBean() : cv.getRootBean(),
                    cv.getPropertyPath().toString(),
                    cv.getInvalidValue(),
                    cv.getMessage()
                );
            }
        }
        return converted;
    }
}
